﻿using System;
using System.Collections.Generic;
using System.Text;
using NUnit.Framework;
using BSM;
using System.Threading;

namespace BSM.Tests
{
    [TestFixture]
    class JobIsRunningMarkerTest
    {
        JobIsRunningMarker testSubject;
        bool functionExecuted;
        bool threadHasStarted;

        [SetUp]
        public void SetUp()
        {
            testSubject = new JobIsRunningMarker("unit test");
            functionExecuted = false;
            threadHasStarted = false;
        }

        [TearDown]
        public void TearDown()
        {
            testSubject.Close();
        }

        [Test]
        public void TestClaimReleaseSingleThread()
        {
            Assert.IsFalse(testSubject.IsRunning);
            testSubject.Claim();
            Assert.IsTrue(testSubject.IsRunning);
            testSubject.Release();
            Assert.IsFalse(testSubject.IsRunning);
        }

        [Test]
        public void TestClaimReleaseMultiThread()
        {
            testSubject.Claim();

            Assert.IsFalse(threadHasStarted);

            Thread newThread = new Thread(new ParameterizedThreadStart(ClaimAndReleaseMarker));
            newThread.Start(testSubject);
            
            while(!threadHasStarted)
            {
                Thread.Sleep(1);
            }
            
            Assert.IsTrue(threadHasStarted);
            Assert.IsFalse(functionExecuted);

            testSubject.Release();

            newThread.Join();

            Assert.IsTrue(functionExecuted);

        }

        public void ClaimAndReleaseMarker(object argument)
        {
            JobIsRunningMarker marker = argument as JobIsRunningMarker;
            threadHasStarted = true;
            marker.Claim();
            functionExecuted = true;
            marker.Release();
        }



    }
}
