﻿using System;
using System.Collections.Generic;
using System.Text;
using NUnit.Framework;
using BSM;
using System.Threading;

namespace BSM.Tests
{
    [TestFixture]
    class JobIsRunningMarkerTestClose
    {
        JobIsRunningMarker testSubject;
        bool functionExecuted;
        bool threadHasStarted;

        [SetUp]
        public void SetUp()
        {
            testSubject = new JobIsRunningMarker("unit test");
            functionExecuted = false;
            threadHasStarted = false;
        }

        [Test]
        public void TestClaimCloseReleaseSingleThread()
        {
            Assert.IsFalse(testSubject.IsRunning);
            testSubject.Claim();

            Assert.IsFalse(testSubject.IsClosed);
            testSubject.Close();
            Assert.IsTrue(testSubject.IsClosed);

            Assert.IsFalse(testSubject.IsRunning);

            Assert.Throws<BSM.core.JobDoesNotExistException>(testSubject.Release);

            Assert.IsFalse(testSubject.IsRunning);
        }

        [Test]
        public void TestClaimCloseReleaseMultiThread()
        {
            testSubject.Claim();

            Assert.IsFalse(threadHasStarted);

            Thread newThread = new Thread(new ParameterizedThreadStart(ClaimAndReleaseMarker));
            newThread.Start(testSubject);

            while (!threadHasStarted)
            {
                Thread.Sleep(1);
            }

            Assert.IsTrue(threadHasStarted);
            Assert.IsFalse(functionExecuted);

            Assert.IsFalse(testSubject.IsClosed);
            testSubject.Close();
            Assert.IsTrue(testSubject.IsClosed);

            newThread.Join();

            Assert.IsTrue(functionExecuted);

        }

        public void ClaimAndReleaseMarker(object argument)
        {
            JobIsRunningMarker marker = argument as JobIsRunningMarker;
            threadHasStarted = true;
            marker.Claim();
            functionExecuted = true;

            Assert.IsTrue(marker.IsClosed);
        }
    }
}
