﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using DataBinding.FormControlValidators;
using DataBinding.Validators;
using DataBinding;

namespace BSM
{
    public partial class NewJobFrame : Form
    {
        private List<IFormControlValidator> validators;
        private DataField<string> jobType;
        private DataField<string> title;
        
        public NewJobFrame(string[] jobTypes, string[] invalidJobTitles)
        {
            if (jobTypes.Length == 0)
                throw new ArgumentException("jobTypes must have at least one type");

            // GUI Designer initilizations
            InitializeComponent();

            //Custom initilizations
            InitializeGUI(jobTypes);
            
            //Initilize fields
            this.validators = new List<IFormControlValidator>();

            DataBinding(invalidJobTitles);
        }

        public string JobType
        {
            get { return jobType.Value; }
        }

        public string Title
        {
            get { return title.Value; }
        }

        private void InitializeGUI(string[] jobTypes)
        {
            this.FormClosing += NewJobFrame_FormClosing;

            this.cmbJobTypes.DataSource = jobTypes;
            this.cmbJobTypes.SelectedIndex = 0;
            //this.txtJobTitle.Text = Properties.Resources.DefaultJobName;

        }

        private void DataBinding(string[] invalidJobTitles)
        {
            this.jobType = new DataField<string>(new NoneEmptyStringValidator(),
                null, this.cmbJobTypes.Text);
            this.title = new DataField<string>(
                new MultiValidator<string>(new IValidator<string>[] {new NoneEmptyStringValidator(), 
                    new ExcludeArrayValidator<string>(invalidJobTitles) }),
                null, String.Empty);

            IFormControlValidator validator;
            validator = new ComboBoxValidatorBase<string>(this.cmbJobTypes,
                jobType, Properties.Resources.InvalidInputJobType);
            validators.Add(validator);

            validator = new TextBoxValidatorColor<string>(this.txtJobTitle,
                title, Properties.Resources.InvalidInputJobTitle, Color.Red);
            validators.Add(validator);
        }

        /// <summary>
        /// Validates all FormControlValidators
        /// Shows error message if any would fail
        /// </summary>
        /// <returns>Returns true if all validators validates, otherwise false</returns>
        private bool ValidateAll()
        {
            foreach (IFormControlValidator validator in validators)
            {
                if (!validator.Validate())
                {
                    MessageBox.Show(validator.GetLastErrorMessage(), Properties.Resources.InvalidInputCaption);
                    return false;
                }
            }
            return true;
        }

        /// <summary>
        /// Form closing event handler.
        /// If the form has been closed with the ok button
        /// all controls are validated.
        /// If validation fails, close is canceled.
        /// </summary>
        private void NewJobFrame_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (this.DialogResult == DialogResult.OK)
                e.Cancel = !ValidateAll();

        }
    }
}