//**********************************//
// Written by: Per Bhlin
// perbo770@student.liu.se
// Copyright: (c) 2009
//
// Implementation started: July 27, 2009
// Last modified: August 12, 2009
//**********************************//

using System;
using System.Collections.Generic;
using System.Text;

namespace BSM.core
{
    /// <summary>
    /// Plugins providing job instances should implement this.
    /// Required.
    /// </summary>
    public interface IJob
    {
        /// <summary>
        /// Job title
        /// </summary>
        string Title { get; set; }

        /// <summary>
        /// Job Type Name, same as plugin name
        /// </summary>
        string TypeName { get; }

        /// <summary>
        /// Enabled property
        /// </summary>
        bool Enabled { get; set; }

        /// <summary>
        /// Run interval, in seconds
        /// </summary>
        int RunIntervalInSeconds { get; set; }

        /// <summary>
        /// Configure the job instance.
        /// May allow user interaction.
        /// </summary>
        /// <param name="owner">owner in case Configre open form</param>
        /// <returns>returns true if changes to the configuration is made</returns>
        bool Configure(System.Windows.Forms.IWin32Window owner);

        /// <summary>
        /// Run job
        /// </summary>
        /// <returns>status</returns>       
        Status Run();

        /// <summary>
        /// Launch GUI or other part of the Application that the job is monitoring.
        /// </summary>
        void Launch();

        /// <summary>
        /// Load Settings to instance from filePath
        /// Throws:
        ///  SettingsException
        /// </summary>
        /// <param name="filePath">file path to settings file</param>     
        void LoadSettings(string filePath);

        /// <summary>
        /// Save settings to file
        /// Throws:
        ///  SettingsException
        /// </summary>
        /// <param name="filePath">path to settings file</param>
        void SaveSettings(string filePath);

        /// <summary>
        /// Delete settings file and do any clean up needed.
        /// Throws:
        ///  SettingsException
        /// </summary>
        /// <param name="filePath">path to settings file</param>
        void DeleteSettings(string filePath);
    }
}
