﻿//**********************************//
// Written by: Per Böhlin
// perbo770@student.liu.se
// Copyright: (c) 2009
//
// Implementation started: July 27, 2009
// Last modified: August 12, 2009
//**********************************//

using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;

namespace BSM.core
{
    /// <summary>
    /// Abstract base class for jobs
    /// Plugins providing job instances should implement this.
    /// Not a requirement.
    /// </summary>
    public abstract class JobBase : IJob
    {
        private string title;
        private readonly string typeName;
        private bool enabled;
        private int runIntervalInSeconds;

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="typeName">The name of the job type</param>
        public JobBase(string typeName)
        {
            this.typeName = typeName;
            title = "";
            enabled = true;
            runIntervalInSeconds = 0;
        }

        /// <summary>
        /// Job title
        /// </summary>
        public string Title
        {
            get { return title; }
            set { title = value; }
        }

        /// <summary>
        /// Job Type Name, same as plugin name
        /// </summary>
        public string TypeName
        {
            get { return typeName; }
        }

        /// <summary>
        /// Enabled property
        /// </summary>
        public bool Enabled
        {
            get { return enabled; }
            set { enabled = value; }
        }

        /// <summary>
        /// Run interval, in seconds
        /// </summary>
        public int RunIntervalInSeconds
        {
            get { return runIntervalInSeconds; }
            set { runIntervalInSeconds = value; }
        }

        /// <summary>
        /// Configure the job instance.
        /// May allow user interaction.
        /// </summary>
        /// <param name="owner">owner in case Configre open form</param>
        /// <returns>returns true if changes to the configuration is made</returns>
        abstract public bool Configure(System.Windows.Forms.IWin32Window owner);

        /// <summary>
        /// Run job
        /// </summary>
        /// <returns>status</returns>
        abstract public Status Run();

        /// <summary>
        /// Launch GUI or other part of the Application that the job is monitoring.
        /// </summary>
        abstract public void Launch();

        /// <summary>
        /// Load Settings to instance from filePath
        /// </summary>
        /// <param name="filePath">file path to settings file</param>
        abstract public void LoadSettings(string filePath);

        /// <summary>
        /// Save settings to file
        /// </summary>
        /// <param name="filePath">path to settings file</param>
        abstract public void SaveSettings(string filePath);

        /// <summary>
        /// Delete settings file and do any clean up needed.
        /// </summary>
        /// <param name="filePath">path to settings file</param>
        abstract public void DeleteSettings(string filePath);
    }
}
