//**********************************//
// Written by: Per Bhlin
// perbo770@student.liu.se
// Copyright: (c) 2009
//
// Implementation started: July 27, 2009
// Last modified: August 12, 2009
//**********************************//

using System;
using System.Collections.Generic;
using System.Text;

namespace BSM.core
{
    /// <summary>
    /// Reflects the status of a job run.
    /// Status Code is used to provide discrete status
    /// Text to provide further info
    /// Date when the status was collected.
    /// </summary>
    public class Status
    {
        public enum StatusCode
        {
            Unknown,
            Green,
            Yellow,
            Red
        }
        
        private StatusCode currentStatus;
        private DateTime date;
        private string text;

        #region Constructors
        /// <summary>
        /// Default constructor
        /// Current status: Unknown
        /// Text: "Unknown"
        /// Date: Now
        /// </summary>
        public Status()
            : this(StatusCode.Unknown)
        {
        }

        /// <summary>
        /// Constructor
        /// Current status: Unknown
        /// Date: Now
        /// </summary>
        /// <param name="text">value of Text property</param>
        public Status(string text)
            : this(StatusCode.Unknown, text)
        {
        }

        /// <summary>
        /// Constructor
        /// Text: status code as string
        /// Date: Now
        /// </summary>
        /// <param name="statusCode">value of Code property</param>
        public Status(StatusCode statusCode)
            : this(statusCode, statusCode.ToString())
        {
        }

        /// <summary>
        /// Constructor
        /// Date: Now
        /// </summary>
        /// <param name="statusCode">value of Code property</param>
        /// <param name="text">value of Text property</param>
        public Status(StatusCode statusCode, string text)
            : this(statusCode, text, DateTime.Now)
        {
        }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="statusCode">value of Code property</param>
        /// <param name="text">value of Text property</param>
        /// <param name="date">value of Date property</param>
        public Status(StatusCode statusCode, string text, DateTime date)
        {
            this.date = DateTime.Now;
            this.currentStatus = statusCode;
            this.text = text;
        }

        /// <summary>
        /// Copy constructor
        /// </summary>
        public Status(Status otherStatus)
        {
            if (otherStatus == null)
                throw new ArgumentException();

            this.date = otherStatus.date;
            this.currentStatus = otherStatus.currentStatus;
            this.text = otherStatus.text;
        }

        #endregion //Constructors

        #region Properties
        /// <summary>
        /// Descrete status code
        /// </summary>
        public StatusCode Code
        {
            get { return currentStatus; }
            set { currentStatus = value; }
        }

        /// <summary>
        /// DateTime of when the status was collected
        /// </summary>
        public DateTime Date
        {
            get { return date; }
            set { date = value; }
        }

        /// <summary>
        /// Further info about the status
        /// </summary>
        public string Text
        {
            get { return text; }
            set { text = value; }
        }
        #endregion //Properties

        #region Methods

        /// <summary>
        /// Compares with other status
        /// If Code or Text are not equal
        /// Status has "changed".
        /// </summary>
        /// <param name="otherStatus">old status compared with</param>
        /// <returns>true if status has changed</returns>
        public bool Changed(Status otherStatus)
        {
            return !this.Code.Equals(otherStatus.Code) ||
                !this.Text.Equals(otherStatus.Text);
        }

        /// <summary>
        /// A field equal comparison.
        /// </summary>
        /// <returns>true if all fields are equal</returns>
        public bool Equals(Status otherStatus)
        {
            return this.currentStatus.Equals(otherStatus.currentStatus) &&
                this.date.Equals(otherStatus.date) &&
                this.text.Equals(otherStatus.text);
        }

        #endregion //Methods
    }
}