﻿//**********************************//
// Written by: Per Böhlin
// perbo770@student.liu.se
// Copyright: (c) 2009
//
// Implementation started: July 27, 2009
// Last modified: August 12, 2009
//**********************************//

using System;
using System.Collections.Generic;
using System.Text;
using System.Xml.Serialization;
using System.IO;
using BSM.core;

namespace BSM
{
    public enum ActionOnClick
    {
        None,
        Launch,
        Configure
    }
    
    [XmlRoot("BsmApplicationSettings")]
    public class BsmApplicationSettings
    {
        private List<JobListing> jobListings;
        private bool running;
        private bool displayMessageOnJobFinished;
        private bool displayMessageOnJobStatusChanged;
        private bool displayMessageOnErrror;
        private ActionOnClick mainFormActionOnClick;
        private ActionOnClick trayActionOnClick;

        public BsmApplicationSettings()
        {
            jobListings = new List<JobListing>();
            running = false;
            displayMessageOnErrror = true;
            displayMessageOnJobFinished = true;
            displayMessageOnJobStatusChanged = true;
            mainFormActionOnClick = ActionOnClick.Launch;
            trayActionOnClick = ActionOnClick.Launch;
        }

        #region Properties

        [XmlElement("DisplayMessageOnJobFinished")]
        public bool DisplayMessageOnJobFinished
        {
            get { return displayMessageOnJobFinished; }
            set { displayMessageOnJobFinished = value; }
        }

        [XmlElement("DisplayMessageOnJobStatusChanged")]
        public bool DisplayMessageOnJobStatusChanged
        {
            get { return displayMessageOnJobStatusChanged; }
            set { displayMessageOnJobStatusChanged = value; }
        }

        [XmlElement("DisplayMessageOnError")]
        public bool DisplayMessageOnError
        {
            get { return displayMessageOnErrror; }
            set { displayMessageOnErrror = value; }
        }
	
        [XmlElement("Running")]
        public bool Running
        {
            get { return running; }
            set { running = value; }
        }

        [XmlElement("MainFormActionOnClick")]
        public ActionOnClick MainFormActionOnClick
        {
            get { return mainFormActionOnClick; }
            set { mainFormActionOnClick = value; }
        }

        [XmlElement("TrayActionOnClick")]
        public ActionOnClick TrayActionOnClick
        {
            get { return trayActionOnClick; }
            set { trayActionOnClick = value; }
        }

        [XmlElement("JobListing")]
        public JobListing[] JobList
        {
            get
            {
                JobListing[] listings = new JobListing[jobListings.Count];
                jobListings.CopyTo(listings);
                return listings;
            }
            set
            {
                if (value == null) return;
                jobListings.Clear();
                jobListings.AddRange((JobListing[])value);
            }
        }

        #endregion Properties

        #region Methods

        public void AddJobListing(JobListing jobListing)
        {
            jobListings.Add(jobListing);
        }

        public void ClearJobListing()
        {
            jobListings.Clear();
        }

        public bool Configure(System.Windows.Forms.IWin32Window owner)
        {
            System.Windows.Forms.FormStartPosition startPosition = (owner == null) ?
                System.Windows.Forms.FormStartPosition.CenterScreen :
                System.Windows.Forms.FormStartPosition.CenterParent;

            ConfigureBsmFrame form =
                new ConfigureBsmFrame(this, startPosition);
            return (form.ShowDialog(owner) == System.Windows.Forms.DialogResult.OK);
 
        }

        static public BsmApplicationSettings LoadFromFile(string filePath)
        {
            try
            {
                if (!File.Exists(filePath))
                    throw new FileNotFoundException(filePath);

                XmlSerializer serializer = new XmlSerializer(typeof(BsmApplicationSettings));
                using (TextReader reader = new StreamReader(filePath))
                {
                    BsmApplicationSettings settings = (BsmApplicationSettings)serializer.Deserialize(reader);
                    reader.Close();
                    return settings;
                }
            }
            catch (FileNotFoundException ex)
            {
                throw new SettingsException(
                    String.Format(
                        Properties.Resources.LoadApplicationSettingsFileMissingFormat,
                        filePath),
                    ex);
            }
            catch (Exception ex)
            {
                throw new SettingsException(Properties.Resources.LoadApplicationSettingsFailedMessage, ex);
            }
        }

        static public void SaveToFile(string filePath, BsmApplicationSettings settings)
        {
            try
            {
                if (!Directory.Exists(Path.GetDirectoryName(filePath)))
                    throw new DirectoryNotFoundException(Path.GetDirectoryName(filePath));

                XmlSerializer serializer = new XmlSerializer(typeof(BsmApplicationSettings));
                using (TextWriter writer = new StreamWriter(filePath))
                {
                    serializer.Serialize(writer, settings);
                    writer.Close();
                }
            }
            catch (DirectoryNotFoundException ex)
            {
                throw new SettingsException(
                    String.Format(
                        Properties.Resources.SaveApplicationSettingsDirectoryMissingFormat,
                        ex.Message),
                    ex);
            }
            catch (Exception ex)
            {
                throw new SettingsException(Properties.Resources.SaveApplicationSettingsFailedMessage, ex);
            }
        }

        #endregion Methods
    }
}
