﻿//**********************************//
// Written by: Per Böhlin
// perbo770@student.liu.se
// Copyright: (c) 2009
//
// Implementation started: July 27, 2009
// Last modified: August 12, 2009
//**********************************//

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace BSM
{
    /// <summary>
    /// Form for configuring the application wide settings
    /// Changes are made to the settings object passed in
    /// through the constructor.
    /// </summary>
    public partial class ConfigureBsmFrame : Form
    {
        private BsmApplicationSettings settings;

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="settings">Application settings object being configured</param>
        /// <param name="startPosition">start up position of the form</param>
        public ConfigureBsmFrame(BsmApplicationSettings settings, 
            FormStartPosition startPosition)
        {
            this.settings = settings;

            //GUI Designer's initializations
            InitializeComponent();

            //Custom initilizations
            InitializeGUI(startPosition);

        }

        /// <summary>
        /// INitialize GUI
        /// </summary>
        /// <param name="startPosition">forms startup position</param>
        private void InitializeGUI(FormStartPosition startPosition)
        {
            chkDisplayJobFinishedMessage.Checked = settings.DisplayMessageOnJobFinished;
            chkDisplayMessageOnJobStatusChange.Checked = settings.DisplayMessageOnJobStatusChanged;
            chkDisplayErrorMessage.Checked = settings.DisplayMessageOnError;

            cmbMainFormActionOnClick.DataSource = Enum.GetNames(typeof(ActionOnClick));
            cmbMainFormActionOnClick.SelectedIndex = (int)settings.MainFormActionOnClick;
            cmbTrayActionOnClick.DataSource = Enum.GetNames(typeof(ActionOnClick));
            cmbTrayActionOnClick.SelectedIndex = (int)settings.TrayActionOnClick;

            this.StartPosition = startPosition;
            this.ShowInTaskbar = (startPosition == FormStartPosition.CenterScreen);
            this.Icon = Properties.Resources.BSM;

            this.FormClosing += new FormClosingEventHandler(ConfigureBsmFrame_FormClosing);
        }

        /// <summary>
        /// EventHandler for the FormClosing event
        /// If user pressed Ok-button, update settings object
        /// </summary>
        void ConfigureBsmFrame_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (this.DialogResult == DialogResult.OK)
            {
                try
                {
                    settings.DisplayMessageOnJobFinished = chkDisplayJobFinishedMessage.Checked;
                    settings.DisplayMessageOnJobStatusChanged = chkDisplayMessageOnJobStatusChange.Checked;
                    settings.DisplayMessageOnError = chkDisplayErrorMessage.Checked;

                    settings.MainFormActionOnClick = (ActionOnClick)cmbMainFormActionOnClick.SelectedIndex;
                    settings.TrayActionOnClick = (ActionOnClick)cmbTrayActionOnClick .SelectedIndex;
                }
                catch (Exception ex)
                {
                    throw new BSM.core.SettingsException(Properties.Resources.ConfigureApplicationErrorMessage, ex);
                }
            }
        }

    }
}