﻿//**********************************//
// Written by: Per Böhlin
// perbo770@student.liu.se
// Copyright: (c) 2009
//
// Implementation started: July 27, 2009
// Last modified: August 12, 2009
//**********************************//

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Reflection;
using System.Resources;

namespace BSM
{
    class DebugReporter
    {
        private TextWriter writer;

        private const string headerFormat = "** {0} **";
        private const string paragraphFormat = "{0}";
        private const string lineFormat = "{0}";
        private const string wordFormat = "{0}";

        public DebugReporter(TextWriter writer)
        {
            this.writer = writer;
        }

        /// <summary>
        /// Writes the Excetuting Assembly's information
        /// </summary>
        public void WriteAssemblyInfo()
        {
            try
            {
                Assembly assembly = Assembly.GetExecutingAssembly();
                AssemblyName name = assembly.GetName();

                this.WriteHeader("Assembly info");
                WriteValuePair("Full name", name.FullName);
                WriteValuePair("Version", name.Version.ToString());

                object[] attributes = assembly.GetCustomAttributes(false);
                foreach (object attrib in attributes)
                {
                    Attribute attribute = attrib as Attribute;
                    if (attribute != null)
                    {
                        WriteLine(attribute.ToString());
                        PropertyInfo[] pi = attribute.GetType().GetProperties(BindingFlags.NonPublic | BindingFlags.Public);
                        WriteProprties(pi);
                    }
                }

                WriteParagraph("");
            }
            catch
            {
                WriteParagraph("An error occured and Assembly Info could not be written correctly");
                WriteParagraph("");
            }
        }

        public void WriteResources(Type type)
        {
            try
            {
                this.WriteHeader("Resources");
                PropertyInfo[] pi = type.GetProperties(BindingFlags.GetProperty | BindingFlags.NonPublic | BindingFlags.Public | BindingFlags.Static);
                WriteProprties(pi);
                WriteParagraph("");
            }
            catch
            {
                WriteParagraph("An error occured and Resources could not be written.");
                WriteParagraph("");
            }
        }

        private void WriteProprties(PropertyInfo[] pi)
        {
            foreach (PropertyInfo prop in pi)
            {
                WriteProperty(prop);
            }
        }

        private void WriteProperty(PropertyInfo prop)
        {
            try
            {
                WriteValuePair(prop.Name, prop.GetValue(null, null).ToString());
            }
            catch
            {
                WriteValuePair(prop.Name, "Error - could not retrieve value");
            }
        }


        public void WriteHeader(string text)
        {
            WriteLine("");
            writer.WriteLine(headerFormat, text);
            WriteUnderScore(text.Length + headerFormat.Length - 3);
        }

        public void WriteParagraph(string text)
        {
            writer.WriteLine(paragraphFormat, text);
            WriteLine("");
        }

        public void WriteLine(string text)
        {
            writer.WriteLine(lineFormat, text);
        }

        public void Write(string text)
        {
            writer.Write(wordFormat, text);
        }

        public void WriteArray(object[] objects, string header)
        {
            WriteHeader(header);
            foreach (object obj in objects)
            {
                WriteValuePair(" ", obj.ToString());
            }
            WriteParagraph("");
        }

        private void WriteUnderScore(string text)
        {
            writer.WriteLine(String.Empty.PadLeft(text.Length, '-'));
        }

        private void WriteUnderScore(int length)
        {
            writer.WriteLine(String.Empty.PadLeft(length, '-'));
        }

        public void WriteValuePair(string title, string value)
        {
            WriteLine(String.Format("{0}: {1}", title, value));
        }
    }
}
