﻿//**********************************//
// Written by: Per Böhlin
// perbo770@student.liu.se
// Copyright: (c) 2009
//
// Implementation started: July 27, 2009
// Last modified: August 12, 2009
//**********************************//

using System;
using BSM.core;
namespace BSM
{
    /// <summary>
    /// Classes implementing IScheduler should 
    /// have the following usage:
    /// 
    /// Close() should be called before left for garbage collection
    /// 
    /// Manage jobs by:
    /// CreateJob(), RemoveJob(), GetJob()
    /// GetJobExists()
    /// 
    /// Query jobs last statuses by:
    /// GetJobStatus()
    /// 
    /// Start/Stop scheduler
    /// to turn on/off automatic job-runs
    /// IsRunning : bool if the scheduler is turned on/off
    /// 
    /// RunJobs() - Run scheduled jobs immediately
    /// 
    /// WaitForAllJobsToFinish()
    /// Halts the execution of the calling thread
    /// until all currently running jobs have finished
    /// 
    /// Properties:
    ///     Jobs : IJob[]
    ///     JobTitles  : string[]
    /// 
    /// Events:
    ///     JobFinished
    ///     StatusChanged
    /// </summary>
    interface IScheduler
    {
        void AddJob(IJob job);
        void RemoveJob(string title);
        
        IJob GetJob(string title);
        bool GetJobExists(string title);
        
        Status GetJobStatus(string title);
        
        IJob[] Jobs { get; }
        string[] JobTitles { get; }

        event JobFinishedEventHandler JobFinished;
        event StatusChangedEventHandler StatusChanged;

        void Start();
        void Stop();
        bool IsRunning { get; }
        void RunJobs();
        void WaitForAllStartedJobsToFinish();

        void Close();
    }
}
