﻿//**********************************//
// Written by: Per Böhlin
// perbo770@student.liu.se
// Copyright: (c) 2009
//
// Implementation started: July 27, 2009
// Last modified: August 12, 2009
//**********************************//

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using DataBinding.FormControlValidators;
using DataBinding.Validators;
using DataBinding;

namespace BSM
{
    public partial class JobFrame : Form
    {
        private List<IFormControlValidator> validators;
        private DataField<string> jobType;
        private DataField<string> title;
        private DataField<int> runInterval;
        
        public JobFrame(string[] jobTypes, string[] invalidJobTitles, bool displayInTray)
        {
            if (jobTypes.Length == 0)
                throw new ArgumentException("jobTypes must have at least one type");

            // GUI Designer initilizations
            InitializeComponent();

            //Custom initilizations
            InitializeGUI(jobTypes, displayInTray);
            
            //Initilize fields
            this.validators = new List<IFormControlValidator>();

            DataBinding(invalidJobTitles);
        }

        public string JobType
        {
            get { return jobType.Value; }
        }

        public string Title
        {
            get { return title.Value; }
        }

        public int RunInterval
        {
            get { return runInterval.Value; }
        }

        public bool DisplayInTray
        {
            get { return chkDisplayInTray.Checked; }
        }

        private void InitializeGUI(string[] jobTypes, bool displayInTray)
        {
            this.FormClosing += JobFrame_FormClosing;

            this.cmbJobTypes.DataSource = jobTypes;
            this.cmbJobTypes.SelectedIndex = 0;
            this.txtJobTitle.Text = "";
            this.txtRunInterval.Text = "";
            this.chkDisplayInTray.Checked = displayInTray;
            this.chkDisplayInTray.Enabled = displayInTray;
        }

        private void DataBinding(string[] invalidJobTitles)
        {
            this.jobType = new DataField<string>(new NoneEmptyStringValidator(),
                null, this.cmbJobTypes.Text);
            this.title = new DataField<string>(
                new JobTitleValidator(invalidJobTitles),
                null, Properties.Resources.DefaultJobName);
            this.runInterval = new DataField<int>(new NoneNegativeValidator<int>(),
                null, Properties.Settings.Default.DefaultRunInterval);

            IFormControlValidator validator;
            validator = new ComboBoxValidatorBase<string>(this.cmbJobTypes,
                jobType, Properties.Resources.InvalidInputJobType);
            validators.Add(validator);

            validator = new TextBoxValidatorColor<string>(this.txtJobTitle,
                title, Properties.Resources.InvalidInputJobTitle, Color.Red);
            validators.Add(validator);

            validator = new TextBoxValidatorColor<int>(this.txtRunInterval,
                runInterval, Properties.Resources.InvalidInputJobRunInterval, Color.Red);
        }

        /// <summary>
        /// Validates all FormControlValidators
        /// Shows error message if any would fail
        /// </summary>
        /// <returns>Returns true if all validators validates, otherwise false</returns>
        private bool ValidateAll()
        {
            foreach (IFormControlValidator validator in validators)
            {
                if (!validator.Validate())
                {
                    MessageBox.Show(validator.GetLastErrorMessage(), Properties.Resources.InvalidInputCaption);
                    return false;
                }
            }
            return true;
        }

        /// <summary>
        /// Form closing event handler.
        /// If the form has been closed with the ok button
        /// all controls are validated.
        /// If validation fails, close is canceled.
        /// </summary>
        private void JobFrame_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (this.DialogResult == DialogResult.OK)
                e.Cancel = !ValidateAll();

        }
    }
}