﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;

namespace BSM
{
    /// <summary>
    /// Thread Safe
    /// Used to mark a job as running
    /// 
    /// </summary>
    class JobIsRunningMarker
    {
        private bool isRunning;
        private bool hasBeenClosed;
        private string title;
        private Semaphore runLock;
        private object waitLock;

        public JobIsRunningMarker(string title)
        {
            this.title = title;
            isRunning = false;
            hasBeenClosed = false;
            waitLock = new object();
            runLock = new Semaphore(1, 1);
        }

        public string JobTitle
        {
            get { return title; }
        }

        public bool IsRunning
        {
            get { return isRunning; }
            set { isRunning = value; }
        }

        public bool IsClosed
        {
            get { return hasBeenClosed; }
        }

        public void Close()
        {
            lock (this)
            {
                if (IsClosed)
                    throw new BSM.core.JobDoesNotExistException(this.title);

                
                hasBeenClosed = true;
                runLock.Close();
                this.isRunning = false;
            }
        }

        public void Claim()
        {
            
            if (IsClosed)
                throw new BSM.core.JobDoesNotExistException(this.title);


            lock (waitLock)
            {
                runLock.WaitOne();
                isRunning = true;
            }
        }

        public void Release()
        {
            if (IsClosed)
                throw new BSM.core.JobDoesNotExistException(this.title);

                isRunning = false;
                runLock.Release();
        }

        public void WaitForJobToFinish()
        {
            lock (waitLock)
            {
                while (isRunning)
                {
                    Thread.Sleep(0);
                }
            }
        }

        public void WaitForJobToFinishAndClose()
        {
            lock (waitLock)
            {
                WaitForJobToFinish();
                Close();
            }
        }
	
    }
}
