﻿using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Reflection;
using BSM.core;

namespace BSM
{
    class PluginLoader
    {
        public static IDictionary<string, IPlugin> Load(string pluginDirectory)
        {
            IDictionary<string, IPlugin> result = new SortedList<string, IPlugin>();

            try
            {
                foreach (Assembly assembly in Assemblies(pluginDirectory))
                {
                    foreach (Type type in ExtractPluginMarkedTypesFromAssembly(assembly))
                    {
                        IPlugin plugin = CreatePluginFromType(type);
                        if (plugin != null)
                        {
                            result.Add(plugin.Name, plugin);
                        }
                    }
                }

            }
            catch (Exception)
            {
                
            }
            return result;
        }

        public static IPlugin CreatePluginFromType(Type type)
        {
            try
            {
                object instance = type.Assembly.CreateInstance(type.ToString());
                return new PluginWrapper(instance);
            }
            catch
            {
                return null;
            }
        }

        private static IEnumerable<Assembly> Assemblies(string filePath)
        {
            Assembly assembly;
            foreach (FileInfo file in new DirectoryInfo(filePath).GetFiles())
            {
                assembly = LoadAssemby(file.FullName);
                if (assembly != null)
                    yield return LoadAssemby(file.FullName);
            }
        }

        private static Assembly LoadAssemby(string filePath)
        {
            if (Path.GetExtension(filePath) == ".dll")
                return Assembly.LoadFrom(filePath);
            else
                return null;
        }


        private static IEnumerable<Type> ExtractPluginMarkedTypesFromAssembly(Assembly assembly)
        {
            foreach (Type type in assembly.GetTypes())
            {
                if (type.GetCustomAttributes(typeof(PluginAttribute), true).Length > 0)
                {
                    yield return type;
                }
            }
        }
    }
}
