﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Reflection;
using BSM.core;

namespace BSM
{
    class PluginWrapper : IPlugin
    {
        object instance;
        Type jobType;
        string name;


        public PluginWrapper(object instance)
        {
            this.instance = instance;

            name = ExtractName(instance);
            if (name == String.Empty || name == null)
                throw new ArgumentException("Type is missing PluginAttribute");

            jobType = ExtractCreateInstanceReturnType(instance);
            if (jobType == null)
                throw new ArgumentException("Type is missing CreateJobAttribute");

            if (!IconPropertyExists(instance))
                throw new ArgumentException("Type is missing Icon Property");
        }

        private string ExtractName(object instance)
        {
            foreach (object attribute in instance.GetType().GetCustomAttributes(typeof(PluginAttribute), true))
            {
                PluginAttribute pluginAttribute = attribute as PluginAttribute;
                if (pluginAttribute != null)
                {
                    return pluginAttribute.Name;
                }
            }
            return null;
        }

        private Type ExtractCreateInstanceReturnType(object instance)
        {
            MethodInfo[] methods = instance.GetType().GetMethods();
            BSM.core.IJob job;
            foreach(MethodInfo method in methods)
            {
                if (method.GetCustomAttributes(typeof(CreateJobAttribute), true).Length > 0)
                {
                    job = (IJob)method.Invoke(instance, null);
                    return job.GetType();
                }
            }
            return null;
        }

        public IJob CreateInstance()
        {
            return (IJob) Activator.CreateInstance(this.jobType);
        }

        public System.Drawing.Icon Icon
        {
            get { return GetIcon(instance); }
        }

        public string Name
        {
            get { return name; }
        }


        private bool IconPropertyExists(object instance)
        {
            PropertyInfo[] properties = instance.GetType().GetProperties();
            foreach (PropertyInfo property in properties)
            {
                if (property.GetCustomAttributes(typeof(IconAttribute), true).Length > 0)
                {
                    return true;
                }
            }
            return false;
        }


        private System.Drawing.Icon GetIcon(object instance)
        {
            PropertyInfo[] properties = instance.GetType().GetProperties();
            foreach (PropertyInfo property in properties)
            {
                if (property.GetCustomAttributes(typeof(IconAttribute), true).Length > 0)
                {
                    return (System.Drawing.Icon) property.GetValue(instance, null);
                }
            }
            return null;
        }

    }
}
