﻿//**********************************//
// Written by: Per Böhlin
// perbo770@student.liu.se
// Copyright: (c) 2009
//
// Implementation started: July 11, 2009
// Last modified: July 23, 2009
//**********************************//

using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using DataBinding;
using DataBinding.FormControlValidators;
using DataBinding.Validators;
using NUnit.Framework;
using System.Drawing;


namespace DataBindingTests.TestCases
{
    [TestFixture]
    public class ComboBoxIndexValidatorColorEnumTest
    {

        public enum Options
        {
            NotAnOption,
            FirstValidOption,
            SecondValidOption,
            InvalidOption
        }

        private ComboBox control;
        private IDataField<Options> dataField;
        private const string message = "Invalid value";
        private const Options expected = Options.FirstValidOption;
        private const Options invalidValue = Options.InvalidOption;
        private IFormControlValidator testSubject;
        private Color expectedValidDataColor;
        private Color expectedInvalidDataColor;

        [SetUp]
        public void SetUp()
        {
            control = new System.Windows.Forms.ComboBox();
            string[] options = Enum.GetNames(typeof(Options));
            foreach (string opt in options)
            {
                control.Items.Add(opt);
            }

            IValidator<Options> validator = new ExcludeValueValidator<Options>(Options.InvalidOption);
            dataField = new DataField<Options>(validator, null);
            expectedValidDataColor = control.ForeColor;
            expectedInvalidDataColor = Color.Red;

            testSubject = new ComboBoxIndexValidatorColor<Options>(control, dataField,
                message, this.expectedInvalidDataColor);
        }
        
        [Test]
        public void TestValidate()
        {
            // Valid value
            control.SelectedIndex = (int)expected;
            Assert.AreEqual(true, dataField.IsValid());
            Assert.AreEqual(true, testSubject.Validate());
            Assert.AreEqual(string.Empty, testSubject.GetLastErrorMessage());

            // Invalid value
            control.SelectedIndex = (int)invalidValue;
            Assert.AreEqual(false, testSubject.Validate());
            Assert.AreEqual(false, dataField.IsValid());
            Assert.AreEqual(message, testSubject.GetLastErrorMessage());
        }

        [Test]
        public void TestChangeOptions()
        {
            // Valid value
            control.SelectedIndex = (int)expected;
            Assert.AreEqual(expected, dataField.Value);
            Assert.AreEqual(true, dataField.IsValid());
            Assert.AreEqual(expectedValidDataColor, control.ForeColor);
            Assert.AreEqual(string.Empty, testSubject.GetLastErrorMessage());

            // Invalid value
            control.SelectedIndex = (int)invalidValue;
            Assert.AreEqual(invalidValue, dataField.Value);
            Assert.AreEqual(false, dataField.IsValid());
            Assert.AreEqual(expectedInvalidDataColor, control.ForeColor);
            Assert.AreEqual(message, testSubject.GetLastErrorMessage());
            Assert.AreEqual(string.Empty, testSubject.GetLastErrorMessage());

        }

        [Test]
        public void TestChangeDataField()
        {
            // Assert init state
            // Valid value
            Assert.AreEqual((int)dataField.Value, control.SelectedIndex);

            // change datafield
            // and assert change propagated to combobox
            dataField.Value = Options.SecondValidOption;
            Assert.AreEqual((int)dataField.Value, control.SelectedIndex);

            // change to invalid value
            // assert textbox changed
            dataField.Value = Options.InvalidOption;
            Assert.AreEqual((int)dataField.Value, control.SelectedIndex);
            Assert.AreEqual(false, dataField.IsValid());
            Assert.AreEqual(expectedInvalidDataColor, control.ForeColor);
            Assert.AreEqual(message, testSubject.GetLastErrorMessage());
            Assert.AreEqual(string.Empty, testSubject.GetLastErrorMessage());

        }
    }
}
