﻿//**********************************//
// Written by: Per Böhlin
// perbo770@student.liu.se
// Copyright: (c) 2009
//
// Implementation started: July 19, 2009
// Last modified: July 23, 2009
//**********************************//

using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using DataBinding;
using DataBinding.FormControlValidators;
using DataBinding.Validators;
using NUnit.Framework;
using System.Drawing;


namespace DataBindingTests.TestCases
{
    [TestFixture]
    public class DateTimePickerValidatorDisplayLabelTest
    {
        private DateTimePicker control;
        private IDataField<DateTime> dataField;
        private const string message = "Invalid value";
        private DateTime expected;
        private DateTime invalidValue;
        private IFormControlValidator testSubject;
        private Label warningLabel;

        [SetUp]
        public void SetUp()
        {
            control = new DateTimePicker();
            warningLabel = new Label();
            expected = DateTime.Now;
            invalidValue = DateTime.Now.AddDays(2);
            IValidator<DateTime> validator = new Mocks.ValidatorMock<DateTime>(invalidValue);
            dataField = new DataField<DateTime>(validator, null, expected);

            testSubject = new DateTimePickerValidatorDisplayLabel(control, dataField,
                message, this.warningLabel);
        }
        
        [Test]
        public void TestValidate()
        {
            // Valid value
            control.Value = expected;
            Assert.AreEqual(true, testSubject.Validate());
            Assert.AreEqual(true, dataField.IsValid());
            Assert.AreEqual(string.Empty, testSubject.GetLastErrorMessage());

            // Invalid value
            control.Value = invalidValue;
            Assert.AreEqual(false, testSubject.Validate());
            Assert.AreEqual(false, dataField.IsValid());
            Assert.AreEqual(message, testSubject.GetLastErrorMessage());
            Assert.AreEqual(String.Empty, testSubject.GetLastErrorMessage());
        }

        [Test]
        public void TestChangeValue()
        {
            // Valid value
            control.Value = DateTime.Now.AddHours(1);   //set to new value
            control.Value = expected;       //so this value triggers a change value event
            Assert.AreEqual(expected, dataField.Value);
            Assert.AreEqual(true, dataField.IsValid());
            Assert.IsFalse(warningLabel.Visible);
            Assert.AreEqual(string.Empty, testSubject.GetLastErrorMessage());

            // Invalid value
            control.Value = invalidValue;
            Assert.AreEqual(invalidValue, dataField.Value);
            Assert.AreEqual(false, dataField.IsValid());
            Assert.IsTrue(warningLabel.Visible);
            Assert.AreEqual(message, testSubject.GetLastErrorMessage());
            Assert.AreEqual(string.Empty, testSubject.GetLastErrorMessage());

        }

        [Test]
        public void TestChangeDataFieldValue()
        {
            // assert init state
            Assert.AreEqual(dataField.Value, control.Value);

            // assert update propagated to textbox
            dataField.Value = DateTime.Now.AddHours(1);
            Assert.AreEqual(dataField.Value, control.Value);

            // set invalid value
            // assert textbox changed
            dataField.Value = invalidValue;
            Assert.AreEqual(dataField.Value, control.Value);
            Assert.AreEqual(false, dataField.IsValid());
            Assert.IsTrue(warningLabel.Visible);
            Assert.AreEqual(message, testSubject.GetLastErrorMessage());
            Assert.AreEqual(string.Empty, testSubject.GetLastErrorMessage());

        }
    }
}
