﻿//**********************************//
// Written by: Per Böhlin
// perbo770@student.liu.se
// Copyright: (c) 2009
//
// Implementation started: July 11, 2009
// Last modified: July 23, 2009
//**********************************//

using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using DataBinding;
using DataBinding.FormControlValidators;
using DataBinding.Validators;
using NUnit.Framework;
using System.Drawing;


namespace DataBindingTests.TestCases
{
    [TestFixture]
    public class TextBoxValidatorColorDoubleTest
    {
        private TextBoxBase textBox;
        private IDataField<double> dataField;
        private const string message = "Invalid value";
        private const double expected = 20.0;
        private const double invalidValue = 0;
        private IFormControlValidator testSubject;
        private Color expectedValidDataColor;
        private Color expectedInvalidDataColor;

        [SetUp]
        public void SetUp()
        {
            textBox = new TextBox();
            IValidator<double> validator = new Mocks.ValidatorMockValueNotZero<double>();
            dataField = new DataField<double>(validator, null);
            expectedValidDataColor = textBox.ForeColor;
            expectedInvalidDataColor = Color.Red;

            testSubject = new TextBoxValidatorColor<double>(textBox, dataField,
                message, this.expectedInvalidDataColor);
        }
        
        [Test]
        public void TestValidate()
        {
            // Valid value
            textBox.Text = expected.ToString();
            Assert.AreEqual(true, testSubject.Validate());
            Assert.AreEqual(true, dataField.IsValid());
            Assert.AreEqual(string.Empty, testSubject.GetLastErrorMessage());

            // Invalid value
            textBox.Text = invalidValue.ToString();
            Assert.AreEqual(false, testSubject.Validate());
            Assert.AreEqual(false, dataField.IsValid());
            Assert.AreEqual(message, testSubject.GetLastErrorMessage());
        }

        [Test]
        public void TestChangeTextBoxText()
        {
            // Valid value
            textBox.Text = expected.ToString();
            Assert.AreEqual(expected, dataField.Value);
            Assert.AreEqual(true, dataField.IsValid());
            Assert.AreEqual(expectedValidDataColor, textBox.ForeColor);
            Assert.AreEqual(string.Empty, testSubject.GetLastErrorMessage());

            // Invalid value
            textBox.Text = invalidValue.ToString();
            Assert.AreEqual(invalidValue, dataField.Value);
            Assert.AreEqual(false, dataField.IsValid());
            Assert.AreEqual(expectedInvalidDataColor, textBox.ForeColor);
            Assert.AreEqual(message, testSubject.GetLastErrorMessage());
            Assert.AreEqual(string.Empty, testSubject.GetLastErrorMessage());

        }

        [Test]
        public void TestChangeDataFieldValue()
        {
            // assert init state
            Assert.AreEqual(dataField.Value, double.Parse(textBox.Text));

            // assert update propagated to textbox
            dataField.Value = 10.0;
            Assert.AreEqual(dataField.Value, double.Parse(textBox.Text));

            // set invalid value
            // assert textbox changed
            dataField.Value = invalidValue;
            Assert.AreEqual(dataField.Value, double.Parse(textBox.Text));
            Assert.AreEqual(false, dataField.IsValid());
            Assert.AreEqual(expectedInvalidDataColor, textBox.ForeColor);
            Assert.AreEqual(message, testSubject.GetLastErrorMessage());
            Assert.AreEqual(string.Empty, testSubject.GetLastErrorMessage());
        }

        [Test]
        public void TestDispose()
        {
            // set invalid value to force color change
            textBox.Text = expected.ToString(); // set to one value and then change it
            textBox.Text = invalidValue.ToString(); // to insure triggering of change event
            Assert.AreEqual(expectedInvalidDataColor, textBox.ForeColor);

            testSubject.Dispose();
            Assert.AreEqual(expectedValidDataColor, textBox.ForeColor);
        }

        [Test]
        public void TestParseFailure()
        {
            //set a valid value and verify propagation
            textBox.Text = expected.ToString();
            Assert.AreEqual(expected, dataField.Value);

            //set invalid value that can not be parsed to double;
            textBox.Text = "g";
            // assert that the invalid value is reverted to last valid value.
            Assert.AreEqual(expected, dataField.Value);
            Assert.AreEqual(expected.ToString(), textBox.Text);
        }
    }
}
