﻿//**********************************//
// Written by: Per Böhlin
// perbo770@student.liu.se
// Copyright: (c) 2009
//
// Implementation started: July 11, 2009
// Last modified: July 23, 2009
//**********************************//

using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using DataBinding;
using DataBinding.FormControlValidators;
using DataBinding.Validators;
using NUnit.Framework;
using System.Drawing;


namespace DataBindingTests.TestCases
{
    [TestFixture]
    public class TextBoxValidatorColorStringTest
    {
        private TextBoxBase textBox;
        private IDataField<string> dataField;
        private const string message = "Invalid value";
        private const string expected = "text";
        private const string invalidValue = "";
        private IFormControlValidator testSubject;
        private Color expectedValidDataColor;
        private Color expectedInvalidDataColor;

        [SetUp]
        public void SetUp()
        {
            textBox = new TextBox();
            IValidator<string> validator = new NoneEmptyStringValidator();
            dataField = new DataField<string>(validator, null);
            expectedValidDataColor = textBox.ForeColor;
            expectedInvalidDataColor = Color.Red;

            testSubject = new TextBoxValidatorColor<string>(textBox, dataField,
                message, this.expectedInvalidDataColor);
        }
        
        [Test]
        public void TestValidate()
        {
            // Valid value
            textBox.Text = expected.ToString();
            Assert.AreEqual(true, testSubject.Validate());
            Assert.AreEqual(true, dataField.IsValid());
            Assert.AreEqual(string.Empty, testSubject.GetLastErrorMessage());

            // Invalid value
            textBox.Text = invalidValue.ToString();
            Assert.AreEqual(false, testSubject.Validate());
            Assert.AreEqual(false, dataField.IsValid());
            Assert.AreEqual(message, testSubject.GetLastErrorMessage());
        }

        [Test]
        public void TestChangeTextBoxText()
        {
            // Valid value
            textBox.Text = expected.ToString();
            Assert.AreEqual(expected, dataField.Value);
            Assert.AreEqual(true, dataField.IsValid());
            Assert.AreEqual(expectedValidDataColor, textBox.ForeColor);
            Assert.AreEqual(string.Empty, testSubject.GetLastErrorMessage());

            // Invalid value
            textBox.Text = invalidValue.ToString();
            Assert.AreEqual(invalidValue, dataField.Value);
            Assert.AreEqual(false, dataField.IsValid());
            Assert.AreEqual(expectedInvalidDataColor, textBox.ForeColor);
            Assert.AreEqual(message, testSubject.GetLastErrorMessage());
            Assert.AreEqual(string.Empty, testSubject.GetLastErrorMessage());

        }
    }
}
