﻿//**********************************//
// Written by: Per Böhlin
// perbo770@student.liu.se
// Copyright: (c) 2009
//
// Implementation started: July 11, 2009
// Last modified: July 23, 2009
//**********************************//

using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;

namespace DataBinding.FormControlValidators
{
    public class ComboBoxValidatorBase<T> : FormControlValidatorBase<T> where T : IComparable
    {

        protected ComboBox comboBox;

        public ComboBoxValidatorBase(ComboBox comboBox,
            IDataField<T> dataField, string invalidValueMessage)
            :
            base(dataField, invalidValueMessage)
        {
            this.comboBox = comboBox;
            comboBox.SelectedItem = dataField.Value;
            this.comboBox.SelectedIndexChanged += this.ComboBox_SelectedIndexChanged;

        }

        override public bool Validate()
        {
            
            UpdateAndValidateValue();
            if (!this.valid)
            {
                this.comboBox.Focus();
            }
            return this.valid;
        }

        virtual protected void ComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            UpdateAndValidateValue();
        }

        virtual protected void UpdateAndValidateValue()
        {
            T value;
            this.valid = GenericParser.Parse<object, T>(comboBox.SelectedItem, out value);
            if (this.valid)
            {
                base.UpdateAndValidateValue(value);
            }
            else //Parse failed, revert to last valid value
            {
                ReverseUpdateAndValidateValue();
            }
        }

        override protected void ReverseUpdateAndValidateValue()
        {
            comboBox.SelectedItem = dataField.Value;
            this.valid = true;
            base.ValidateValue();
        }

        override public void Dispose()
        {
            base.Dispose();
            this.comboBox.SelectedIndexChanged -= this.ComboBox_SelectedIndexChanged;
        }
    }
}
