﻿//**********************************//
// Written by: Per Böhlin
// perbo770@student.liu.se
// Copyright: (c) 2009
//
// Implementation started: July 11, 2009
// Last modified: July 23, 2009
//**********************************//

using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Drawing;

namespace DataBinding.FormControlValidators
{
    public class ControlTextValidatorBase<T> : FormControlValidatorBase<T> where T : IComparable
    {
        protected Control control;

        // constructor
        public ControlTextValidatorBase(Control control, IDataField<T> dataField, 
            string invalidValueMessage) :
            base(dataField, invalidValueMessage)
        {
            this.control = control;
            // update textbox so values match before adding event handler
            this.control.Text = dataField.ToString();
            this.control.TextChanged += this.Control_TextChanged;
        }

        /// <summary>
        /// Updates the field with the value of the control and validates it
        /// Returns false on fail.
        /// </summary>
        /// <returns>returns true if valid value in the control</returns>
		override public bool Validate()
		{
            UpdateAndValidateValue();
            return this.valid;
		}

        virtual protected void Control_TextChanged(object sender, EventArgs e)
        {
            UpdateAndValidateValue();
        }

        protected void UpdateAndValidateValue()
        {
            T value;
            this.valid = GenericParser.Parse<string, T>(control.Text, out value);
            if (this.valid)
            {
                base.UpdateAndValidateValue(value);
            }
            else //if parse failed, revert to last valid value
            {
                ReverseUpdateAndValidateValue();
            }
        }

        override protected void ReverseUpdateAndValidateValue()
        {
            control.Text = dataField.ToString();
            this.valid = true;
            base.ValidateValue();
        }

        override public void Dispose()
        {
            base.Dispose();
            this.control.TextChanged -= this.Control_TextChanged;
        }
    }
}
