﻿//**********************************//
// Written by: Per Böhlin
// perbo770@student.liu.se
// Copyright: (c) 2009
//
// Implementation started: July 19, 2009
// Last modified: July 23, 2009
//**********************************//

using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;

namespace DataBinding.FormControlValidators
{
    public class DateTimePickerValidatorBase : FormControlValidatorBase<DateTime>
    {
        protected DateTimePicker control;
        
        public DateTimePickerValidatorBase(DateTimePicker control,
            IDataField<DateTime> dataField, string invalidValueMessage)
            : base(dataField, invalidValueMessage)
        {
            this.control = control;
            this.control.Value = dataField.Value;
            this.control.ValueChanged += this.Control_ValueChanged;
        }

        public override bool Validate()
        {
            UpdateAndValidateValue();
            if (!this.valid)
            {
                this.control.Focus();
            }
            return this.valid;
        }

        virtual protected void Control_ValueChanged(object sender, EventArgs e)
        {
            UpdateAndValidateValue();
        }

        protected void UpdateAndValidateValue()
        {
            DateTime value;
            this.valid = GenericParser.Parse<DateTime, DateTime>(control.Value, out value);
            base.UpdateAndValidateValue(value);
        }

        override protected void ReverseUpdateAndValidateValue()
        {
            control.Value = dataField.Value;
            this.valid = true;
            base.ValidateValue();
        }

        override public void Dispose()
        {
            base.Dispose();
            this.control.ValueChanged -= this.Control_ValueChanged;
        }
    }
}
