﻿//**********************************//
// Written by: Per Böhlin
// perbo770@student.liu.se
// Copyright: (c) 2009
//
// Implementation started: July 27, 2009
// Last modified: August 12, 2009
//**********************************//

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using DataBinding.FormControlValidators;

namespace DataBinding
{
    public partial class InputBox<T> : Form where T : IComparable
    {
        static Form form;

        public static DialogResult Show(IDataField<T> field, 
            string invalidValueMessage,
            string caption, 
            string prompt)
        {
            form = new InputBox<T>(field, invalidValueMessage, caption, prompt);
            return form.ShowDialog();
        }

        IDataField<T> field;
        IFormControlValidator validator;

        protected InputBox(IDataField<T> field, string invalidValueMessage, 
            string caption, string prompt)
        {
            // GUI designer
            InitializeComponent();

            this.Text = caption;
            this.lblPrompt.Text = prompt;

            this.FormClosing += InputBox_FormClosing;

            validator = new TextBoxValidatorColor<T>(this.txtValue, 
                field, invalidValueMessage, Color.Red);
            this.field = field;
            this.field.Changed += Field_ValueChanged;
        }

        private void Field_ValueChanged(object sender, DataChangedEventArgs<T> args)
        {
            if (!validator.Validate())
            {
                this.erpInvalidValue.SetError(txtValue, validator.GetLastErrorMessage());
                
            }
            else
            {
                this.erpInvalidValue.Clear();
            }
        }

        private void InputBox_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (this.DialogResult == DialogResult.OK)
            {
                if (!validator.Validate())
                {
                    MessageBox.Show(validator.GetLastErrorMessage());
                    e.Cancel = true;
                }
            }
        }

    }
}