﻿//**********************************//
// Written by: Per Böhlin
// perbo770@student.liu.se
// Copyright: (c) 2009
//
// Implementation started: July 11, 2009
// Last modified: July 23, 2009
//**********************************//

using System;
using System.Collections.Generic;
using System.Text;

namespace DataBinding.FormControlValidators
{
    /// <summary>
    /// Abstact base class for SimpleFormControlValidators
    /// </summary>
    public abstract class SimpleFormControlValidatorBase : IFormControlValidator
    {
        protected string invalidMessage;
        protected bool valid;
        
        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="dataField">Data field</param>
        /// <param name="invalidMessage"> message to be displayed if explcit validation fails</param>
        public SimpleFormControlValidatorBase(string invalidMessage)
        {
            this.invalidMessage = invalidMessage;
            this.valid = false;
        }

        /// <summary>
        /// Abstract method.
        /// Should return true if value parsing between control and field succeeded and
        /// value is valid.
        /// Subclass implementation should call UpdateAndValidateValue(T value)
        /// </summary>
        /// <returns></returns>
        public abstract bool Validate();

        /// <summary>
        /// Returns error mesage if an error has occured.
        /// Otherwise empty string
        /// </summary>
        /// <returns>error message or empty string</returns>
        public string GetLastErrorMessage()
        {
            if (this.valid)
                return string.Empty;

            this.valid = true;
            return invalidMessage;
        }

        virtual public void Dispose()
        {
        }
    }
}
