﻿//**********************************//
// Written by: Per Böhlin
// perbo770@student.liu.se
// Copyright: (c) 2009
//
// Implementation started: July 27, 2009
// Last modified: August 12, 2009
//**********************************//

using System;
using System.Collections.Generic;
using System.Text;
using BSM.core;
using System.Xml.Serialization;

namespace Hudson
{
    /// <summary>
    /// Hudson Settings
    /// Holds all settings unique for a Hudson Job.
    /// Supports Xml Serialization
    /// </summary>
    [XmlRoot("Settings")]
    public class HudsonSettings
    {
        public enum AuthenticationMethods
        {
            None = 0,
            Basic
        }

        private string urlFeed;
        private string launch;
        private int timeout;
        private AuthenticationMethods authenticationMethod;
        private string userName;
        private string password;
        private string filterPattern;

        /// <summary>
        /// Default Constructor
        /// </summary>
        public HudsonSettings()
        {
            urlFeed = Properties.Settings.Default.DefaultUrlFeed;
            launch = Properties.Settings.Default.DefaultLaunch;
            authenticationMethod = AuthenticationMethods.None;
            userName = String.Empty;
            password = String.Empty;
            filterPattern = String.Empty;
            timeout = Properties.Settings.Default.DefaultTimeout;
        }

        /// <summary>
        /// Copy constructor
        /// </summary>
        public HudsonSettings(HudsonSettings otherSettings)
        {
            this.UrlFeed = otherSettings.UrlFeed;
            this.launch = otherSettings.Launch;
            this.Timeout = otherSettings.Timeout;
            this.AuthenticationMethod = otherSettings.AuthenticationMethod;
            this.UserName = otherSettings.UserName;
            this.Password = otherSettings.Password;
            this.FilterPattern = otherSettings.FilterPattern;
        }

        #region Properties

        [XmlElement("urlFeed")]
        public string UrlFeed
        {
            get { return urlFeed; }
            set { urlFeed = value; }
        }

        [XmlElement("launch")]
        public string Launch
        {
            get { return launch; }
            set { launch = value; }
        }

        [XmlElement("timeout")]
        public int Timeout
        {
            get { return timeout; }
            set { timeout = value; }
        }

        [XmlElement("authenticationmethod")]
        public AuthenticationMethods AuthenticationMethod
        {
            get { return authenticationMethod; }
            set { authenticationMethod = value; }
        }

        [XmlElement("username")]
        public string UserName
        {
            get { return userName; }
            set { userName = value; }
        }

        [XmlElement("password")]
        public string Password
        {
            get { return password; }
            set { password = value; }
        }

        [XmlElement("filterpattern")]
        public string FilterPattern
        {
            get { return filterPattern; }
            set { filterPattern = value; }
        }

        #endregion //Properties
    }
}
